# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps

#  pylint: disable=line-too-long


def add_helps(command_group, server_type):
    helps['{}'.format(command_group)] = """
                type: group
                short-summary: Manage Azure Database for {} servers.
                """.format(server_type)
    helps['{} server'.format(command_group)] = """
                type: group
                short-summary: Manage {} servers.
                """.format(server_type)
    helps['{} server create'.format(command_group)] = """
                type: command
                short-summary: Create a server.
                examples:
                    - name: Create a {0} server with only required paramaters in North Europe.
                      text: az {1} server create -l northeurope -g testgroup -n testsvr -u username -p password
                    - name: Create a {0} server with a Standard performance tier and 2 vcore in North Europe.
                      text: az {1} server create -l northeurope -g testgroup -n testsvr -u username -p password \\
                            --sku-name GP_Gen4_2
                    - name: Create a {0} server with all paramaters set.
                      text: az {1} server create -l northeurope -g testgroup -n testsvr -u username -p password \\
                            --sku-name B_Gen4_2 --ssl-enforcement Disabled \\
                            --storage-size 51200 --tags "key=value" --version {{server-version}}
                """.format(server_type, command_group)
    helps['{} server restore'.format(command_group)] = """
                type: command
                short-summary: Restore a server from backup.
                examples:
                    - name: Restore 'testsvr' as 'testsvrnew'.
                      text: az {0} server restore -g testgroup -n testsvrnew --source-server testsvr --restore-point-in-time "2017-06-15T13:10:00Z"
                    - name: Restore 'testsvr2' to 'testsvrnew', where 'testsvrnew' is in a different resource group than the backup.
                      text: |
                        az {0} server restore -g testgroup -n testsvrnew \\
                            -s "/subscriptions/${{SubID}}/resourceGroups/${{ResourceGroup}}/providers/Microsoft.DBfor{1}/servers/testsvr2" \\
                            --restore-point-in-time "2017-06-15T13:10:00Z"
                """.format(command_group, server_type)
    helps['{} server georestore'.format(command_group)] = """
                type: command
                short-summary: Georestore a server from backup.
                examples:
                    - name: Georestore 'testsvr' as 'testsvrnew' where 'testsvrnew' is in same resource group as 'testsvr'.
                      text: az {0} server georestore -g testgroup -n testsvrnew --source-server testsvr -l westus2
                    - name: Georestore 'testsvr2' to 'testsvrnew', where 'testsvrnew' is in the different resource group as the original server.
                      text: |
                        az {0} server georestore -g testgroup -n testsvrnew \\
                            -s "/subscriptions/${{SubID}}/resourceGroups/${{ResourceGroup}}/providers/Microsoft.DBfor{1}/servers/testsvr2"
                            -l westus2 --sku-name GP_Gen5_2
                """.format(command_group, server_type)
    helps['{} server update'.format(command_group)] = """
                type: command
                short-summary: Update a server.
                examples:
                    - name: Update a server's sku.
                      text: az {0} server update -g testgroup -n testsvrnew --sku-name GP_Gen4_4
                    - name: Update a server's tags.
                      text: az {0} server update -g testgroup -n testsvrnew --tags "k1=v1" "k2=v2"
                """.format(command_group)
    helps['{} server wait'.format(command_group)] = """
                type: command
                short-summary: Wait for server to satisfy certain conditions.
                """
    helps['{} server delete'.format(command_group)] = """
                type: command
                short-summary: Delete a server.
                """
    helps['{} server show'.format(command_group)] = """
                type: command
                short-summary: Get the details of a server.
                """
    helps['{} server list'.format(command_group)] = """
                type: command
                short-summary: List available servers.
                examples:
                    - name: List all {0} servers in a subscription.
                      text: az {1} server list
                    - name: List all {0} servers in a resource group.
                      text: az {1} server list -g testgroup
                """.format(server_type, command_group)
    helps['{} server firewall-rule'.format(command_group)] = """
                type: group
                short-summary: Manage firewall rules for a server.
                """
    helps['{} server firewall-rule create'.format(command_group)] = """
                type: command
                short-summary: Create a new firewall rule for a server.
                examples:
                    - name: Create a firewall rule allowing all connections from all IP addresses.
                      text: az {} server firewall-rule create -g testgroup -s testsvr -n allowall --start-ip-address 0.0.0.0 --end-ip-address 255.255.255.255
                """.format(command_group)
    helps['{} server firewall-rule update'.format(command_group)] = """
                type: command
                short-summary: Update a firewall rule.
                examples:
                    - name: Update a firewall rule's start IP address.
                      text: az {0} server firewall-rule update -g testgroup -s testsvr -n allowall --start-ip-address 0.0.0.1
                    - name: Update a firewall rule's start and end IP address.
                      text: az {0} server firewall-rule update -g testgroup -s testsvr -n allowall --start-ip-address 0.0.0.1 --end-ip-address 255.255.255.254
                """.format(command_group)
    helps['{} server firewall-rule delete'.format(command_group)] = """
                type: command
                short-summary: Delete a firewall rule.
                """
    helps['{} server firewall-rule show'.format(command_group)] = """
                type: command
                short-summary: Get the details of a firewall rule.
                """
    helps['{} server firewall-rule list'.format(command_group)] = """
                type: command
                short-summary: List all firewall rules for a server.
                """
    helps['{} server vnet-rule'.format(command_group)] = """
                type: group
                short-summary: Manage a server's virtual network rules.
                """
    helps['{} server vnet-rule update'.format(command_group)] = """
                type: command
                short-summary: Update a virtual network rule.
                """
    helps['{} server vnet-rule create'.format(command_group)] = """
                type: command
                short-summary: Create a virtual network rule to allows access to a {} server.
                examples:
                    - name: Create a virtual network rule by providing the subnet id.
                      text: az {} server vnet-rule create -g testgroup -s testsvr -n vnetRuleName --subnet /subscriptions/{{SubID}}/resourceGroups/{{ResourceGroup}}/providers/Microsoft.Network/virtualNetworks/vnetName/subnets/subnetName
                    - name: Create a vnet rule by providing the vnet and subnet name. The subnet id is created by taking the resource group name and subscription id of the server.
                      text: az {} server vnet-rule create -g testgroup -s testsvr -n vnetRuleName --subnet subnetName --vnet-name vnetName
                """.format(server_type, command_group, command_group)
    helps['{} server configuration'.format(command_group)] = """
                type: group
                short-summary: Manage configuration values for a server.
                """
    helps['{} server configuration set'.format(command_group)] = """
                type: command
                short-summary: Update the configuration of a server.
                examples:
                    - name: Set a new configuration value.
                      text: az {0} server configuration set -g testgroup -s testsvr -n {{config_name}} --value {{config_value}}
                    - name: Set a configuration value to its default.
                      text: az {0} server configuration set -g testgroup -s testsvr -n {{config_name}}
                """.format(command_group)
    helps['{} server configuration show'.format(command_group)] = """
                type: command
                short-summary: Get the configuration for a server."
                """
    helps['{} server configuration list'.format(command_group)] = """
                type: command
                short-summary: List the configuration values for a server.
                """
    helps['{} server-logs'.format(command_group)] = """
                type: group
                short-summary: Manage server logs.
                """
    helps['{} server-logs list'.format(command_group)] = """
                type: command
                short-summary: List log files for a server.
                examples:
                    - name: List log files for 'testsvr' modified in the last 72 hours (default value).
                      text: az {0} server-logs list -g testgroup -s testsvr
                    - name: List log files for 'testsvr' modified in the last 10 hours.
                      text: az {0} server-logs list -g testgroup -s testsvr --file-last-written 10
                    - name: List log files for 'testsvr' less than 30Kb in size.
                      text: az {0} server-logs list -g testgroup -s testsvr --max-file-size 30
                """.format(command_group)
    helps['{} server-logs download'.format(command_group)] = """
                type: command
                short-summary: Download log files.
                examples:
                    - name: Download log files f1 and f2 to the current directory from the server 'testsvr'.
                      text: az {} server-logs download -g testgroup -s testsvr -n f1.log f2.log
                """.format(command_group)
    helps['{} db'.format(command_group)] = """
                type: group
                short-summary: Manage {0} databases on a server.
                """.format(server_type)
    helps['{} db create'.format(command_group)] = """
                type: command
                short-summary: Create a {0} database.
                examples:
                    - name: Create database 'testdb' in the server 'testsvr' with the default parameters.
                      text: az {1} db create -g testgroup -s testsvr -n testdb
                    - name: Create database 'testdb' in server 'testsvr' with a given character set and collation rules.
                      text: az {1} db create -g testgroup -s testsvr -n testdb --charset {{valid_charset}} --collation {{valid_collation}}
                """.format(server_type, command_group)
    helps['{} db delete'.format(command_group)] = """
                type: command
                short-summary: Delete a database.
                examples:
                    - name: Delete database 'testdb' in the server 'testsvr'.
                      text: az {0} db delete -g testgroup -s testsvr -n testdb
                """.format(command_group)
    helps['{} db show'.format(command_group)] = """
                type: command
                short-summary: Show the details of a database.
                examples:
                    - name: Show database 'testdb' in the server 'testsvr'.
                      text: az {0} db show -g testgroup -s testsvr -n testdb
                """.format(command_group)
    helps['{} db list'.format(command_group)] = """
                type: command
                short-summary: List the databases for a server.
                examples:
                    - name: List databases in the server 'testsvr'.
                      text: az {0} db list -g testgroup -s testsvr
                """.format(command_group)


add_helps("mysql", "MySQL")
add_helps("postgres", "PostgreSQL")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType

from azure.cli.command_modules.rdbms._client_factory import (
    cf_mysql_servers,
    cf_mysql_db,
    cf_mysql_firewall_rules,
    cf_mysql_virtual_network_rules_operations,
    cf_mysql_config,
    cf_mysql_log,
    cf_postgres_servers,
    cf_postgres_db,
    cf_postgres_firewall_rules,
    cf_postgres_virtual_network_rules_operations,
    cf_postgres_config,
    cf_postgres_log)


# pylint: disable=too-many-locals, too-many-statements, line-too-long
def load_command_table(self, _):

    rdbms_custom = CliCommandType(operations_tmpl='azure.cli.command_modules.rdbms.custom#{}')

    mysql_servers_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.mysql.operations.servers_operations#ServersOperations.{}',
        client_factory=cf_mysql_servers
    )

    postgres_servers_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.postgresql.operations.servers_operations#ServersOperations.{}',
        client_factory=cf_postgres_servers
    )

    mysql_firewall_rule_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.mysql.operations.firewall_rules_operations#FirewallRulesOperations.{}',
        client_factory=cf_mysql_firewall_rules
    )

    postgres_firewall_rule_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.postgresql.operations.firewall_rules_operations#FirewallRulesOperations.{}',
        client_factory=cf_postgres_firewall_rules
    )

    mysql_vnet_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.mysql.operations.virtual_network_rules_operations#VirtualNetworkRulesOperations.{}',
        client_factory=cf_mysql_virtual_network_rules_operations
    )

    postgres_vnet_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.postgresql.operations.virtual_network_rules_operations#VirtualNetworkRulesOperations.{}',
        client_factory=cf_postgres_virtual_network_rules_operations
    )

    mysql_config_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.mysql.operations.configurations_operations#ConfigurationsOperations.{}',
        client_factory=cf_mysql_config
    )

    postgres_config_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.postgresql.operations.configurations_operations#ConfigurationsOperations.{}',
        client_factory=cf_postgres_config
    )

    mysql_log_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.mysql.operations.log_files_operations#LogFilesOperations.{}',
        client_factory=cf_mysql_log
    )

    postgres_log_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.postgresql.operations.log_files_operations#LogFilesOperations.{}',
        client_factory=cf_postgres_log
    )

    mysql_db_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.mysql.operations.databases_operations#DatabasesOperations.{}',
        client_factory=cf_mysql_db
    )

    postgres_db_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.rdbms.postgresql.operations.databases_operations#DatabasesOperations.{}',
        client_factory=cf_postgres_db
    )

    with self.command_group('mysql server', mysql_servers_sdk, client_factory=cf_mysql_servers) as g:
        g.custom_command('create', '_server_create')
        g.custom_command('restore', '_server_restore', supports_no_wait=True)
        g.custom_command('georestore', '_server_georestore', supports_no_wait=True)
        g.command('delete', 'delete', confirmation=True)
        g.show_command('show', 'get')
        g.custom_command('list', '_server_list_custom_func')
        g.generic_update_command('update',
                                 getter_name='_server_update_get', getter_type=rdbms_custom,
                                 setter_name='_server_update_set', setter_type=rdbms_custom, setter_arg_name='parameters',
                                 custom_func_name='_server_update_custom_func')
        g.custom_wait_command('wait', '_server_mysql_get')

    with self.command_group('postgres server', postgres_servers_sdk, client_factory=cf_postgres_servers) as g:
        g.custom_command('create', '_server_create')
        g.custom_command('restore', '_server_restore', supports_no_wait=True)
        g.custom_command('georestore', '_server_georestore', supports_no_wait=True)
        g.command('delete', 'delete', confirmation=True)
        g.show_command('show', 'get')
        g.custom_command('list', '_server_list_custom_func')
        g.generic_update_command('update',
                                 getter_name='_server_update_get', getter_type=rdbms_custom,
                                 setter_name='_server_update_set', setter_type=rdbms_custom, setter_arg_name='parameters',
                                 custom_func_name='_server_update_custom_func')
        g.custom_wait_command('wait', '_server_postgresql_get')

    with self.command_group('mysql server firewall-rule', mysql_firewall_rule_sdk) as g:
        g.command('create', 'create_or_update')
        g.command('delete', 'delete', confirmation=True)
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')
        g.generic_update_command('update',
                                 getter_name='_firewall_rule_custom_getter', getter_type=rdbms_custom,
                                 setter_name='_firewall_rule_custom_setter', setter_type=rdbms_custom, setter_arg_name='parameters',
                                 custom_func_name='_firewall_rule_update_custom_func')

    with self.command_group('postgres server firewall-rule', postgres_firewall_rule_sdk) as g:
        g.command('create', 'create_or_update')
        g.command('delete', 'delete', confirmation=True)
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')
        g.generic_update_command('update',
                                 getter_name='_firewall_rule_custom_getter', getter_type=rdbms_custom,
                                 setter_name='_firewall_rule_custom_setter', setter_type=rdbms_custom, setter_arg_name='parameters',
                                 custom_func_name='_firewall_rule_update_custom_func')

    with self.command_group('mysql server vnet-rule', mysql_vnet_sdk) as g:
        g.command('create', 'create_or_update')
        g.command('delete', 'delete')
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')
        g.generic_update_command('update')

    with self.command_group('postgres server vnet-rule', postgres_vnet_sdk) as g:
        g.command('create', 'create_or_update')
        g.command('delete', 'delete')
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')
        g.generic_update_command('update')

    with self.command_group('mysql server configuration', mysql_config_sdk) as g:
        g.command('set', 'create_or_update')
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')

    with self.command_group('postgres server configuration', postgres_config_sdk) as g:
        g.command('set', 'create_or_update')
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')

    with self.command_group('mysql server-logs', mysql_log_sdk, client_factory=cf_mysql_log) as g:
        g.custom_command('list', '_list_log_files_with_filter')
        g.custom_command('download', '_download_log_files')

    with self.command_group('postgres server-logs', postgres_log_sdk, client_factory=cf_postgres_log) as g:
        g.custom_command('list', '_list_log_files_with_filter')
        g.custom_command('download', '_download_log_files')

    with self.command_group('mysql db', mysql_db_sdk) as g:
        g.command('create', 'create_or_update')
        g.command('delete', 'delete', confirmation=True)
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')

    with self.command_group('postgres db', postgres_db_sdk) as g:
        g.command('create', 'create_or_update')
        g.command('delete', 'delete', confirmation=True)
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import AzArgumentContext


class RdbmsArgumentContext(AzArgumentContext):  # pylint: disable=too-few-public-methods

    def __init__(self, command_loader, scope, **kwargs):    # pylint: disable=unused-argument
        super(RdbmsArgumentContext, self).__init__(command_loader, scope)
        self.validators = []

    def expand(self, dest, model_type, group_name=None, patches=None):
        super(RdbmsArgumentContext, self).expand(dest, model_type, group_name, patches)

        from knack.arguments import ignore_type

        # Remove the validator and store it into a list
        arg = self.command_loader.argument_registry.arguments[self.command_scope].get(dest, None)
        if not arg:  # when the argument context scope is N/A
            return

        self.validators.append(arg.settings['validator'])
        dest_option = ['--__{}'.format(dest.upper())]
        if dest == 'parameters':
            from .validators import get_combined_validator
            self.argument(dest,
                          arg_type=ignore_type,
                          options_list=dest_option,
                          validator=get_combined_validator(self.validators))
        else:
            self.argument(dest, options_list=dest_option, arg_type=ignore_type, validator=None)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.prompting import prompt_pass, NoTTYException
from knack.util import CLIError

from azure.cli.core.commands.validators import (
    get_default_location_from_resource_group, validate_tags)


def get_combined_validator(validators):
    def _final_validator_impl(cmd, namespace):
        # do additional creation validation
        verb = cmd.name.rsplit(' ', 1)[1]
        if verb == 'create':
            password_validator(namespace)
            get_default_location_from_resource_group(cmd, namespace)

        validate_tags(namespace)

        for validator in validators:
            validator(namespace)

    return _final_validator_impl


def configuration_value_validator(ns):
    val = ns.value
    if val is None or not val.strip():
        ns.value = None
        ns.source = 'system-default'


def password_validator(ns):
    if not ns.administrator_login_password:
        try:
            ns.administrator_login_password = prompt_pass(msg='Admin Password: ')
        except NoTTYException:
            raise CLIError('Please specify password in non-interactive mode.')
